'use strict';

/**
 * Module to count up
 */
var Counter = function () 
{
    this.distribution = new (require('distribute-js'))();
    this.number = 0;
    this.display = 0;
    this.time = 1000;
    this.animated = false;
    this._onTick = function () {};
    this._onEnd = function () {};
};

module.exports = Counter;

/**
 * Method to set the number to count to
 * @param  {integer} number
 * @return {Counter}
 */
Counter.prototype.count = function (number)
{
    this.number = number;
    return this;
};

/**
 * Method to enable animated counting
 * @return {Counter}
 */
Counter.prototype.animate = function ()
{
    this.animated = true;
    return this;
};

/**
 * Method to set the time in which the counting should be done
 * @param  {integer} milliseconds
 * @return {Counter}
 */
Counter.prototype.in = function (milliseconds)
{
    this.time = milliseconds;
    return this;
};

/**
 * Method to trigger a tick callback that processes a countup
 * @return {Counter}
 */
Counter.prototype.tick = function ()
{
    this.display++;
    this._onTick(this.display);    
    return this;
};

/**
 * Method to set the `onUpdate` callback which will be triggered on each countup
 * @param  {function} callback
 * @return {Counter}
 */
Counter.prototype.onUpdate = function (callback) {
    if (typeof callback !== 'function') {
        return console.error('`onUpdate` callback needs to be a function.');
    }
    this._onTick = callback;
    return this;
};

/**
 * Method to set the `onDone` callback which will be triggered after the counting goal is reached
 * @param  {Function} callback
 * @return {Counter}
 */
Counter.prototype.onDone = function (callback) {
    if (typeof callback !== 'function') {
        return console.error('`onDone` callback needs to be a function.');
    }

    this._onEnd = callback;
    return this;
}

/**
 * Method to start the coutup
 */
Counter.prototype.run = function ()
{
    if (this.animated !== true) {
        this.display = this.number - 1;
        this.tick();
    } else {
        for (let counter = 0; counter < this.number; counter++) {        
            this.distribution.add(this.tick.bind(this));
        }
        let promise = this.distribution.in(this.time).run();
        promise.resolve(this._onEnd);
    }
};